<?php
declare(strict_types=1);

namespace CakeProducts\Model\Table;

use ArrayObject;
use Cake\Core\Configure;
use Cake\Datasource\EntityInterface;
use Cake\Datasource\ResultSetInterface;
use Cake\Event\EventInterface;
use Cake\ORM\Association\BelongsTo;
use Cake\ORM\Behavior\TimestampBehavior;
use Cake\ORM\Query\SelectQuery;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use CakeProducts\Model\Entity\ExternalProductCatalogsProductCatalog;
use Closure;
use Psr\SimpleCache\CacheInterface;

/**
 * ExternalProductCatalogsProductCatalogs Model
 *
 * @property ExternalProductCatalogsTable&BelongsTo $ExternalProductCatalogs
 * @property ProductCatalogsTable&BelongsTo $ProductCatalogs
 *
 * @method ExternalProductCatalogsProductCatalog newEmptyEntity()
 * @method ExternalProductCatalogsProductCatalog newEntity(array $data, array $options = [])
 * @method array<ExternalProductCatalogsProductCatalog> newEntities(array $data, array $options = [])
 * @method ExternalProductCatalogsProductCatalog get(mixed $primaryKey, array|string $finder = 'all', CacheInterface|string|null $cache = null, Closure|string|null $cacheKey = null, mixed ...$args)
 * @method ExternalProductCatalogsProductCatalog findOrCreate($search, ?callable $callback = null, array $options = [])
 * @method ExternalProductCatalogsProductCatalog patchEntity(EntityInterface $entity, array $data, array $options = [])
 * @method array<ExternalProductCatalogsProductCatalog> patchEntities(iterable $entities, array $data, array $options = [])
 * @method ExternalProductCatalogsProductCatalog|false save(EntityInterface $entity, array $options = [])
 * @method ExternalProductCatalogsProductCatalog saveOrFail(EntityInterface $entity, array $options = [])
 * @method iterable<ExternalProductCatalogsProductCatalog>|ResultSetInterface<ExternalProductCatalogsProductCatalog>|false saveMany(iterable $entities, array $options = [])
 * @method iterable<ExternalProductCatalogsProductCatalog>|ResultSetInterface<ExternalProductCatalogsProductCatalog> saveManyOrFail(iterable $entities, array $options = [])
 * @method iterable<ExternalProductCatalogsProductCatalog>|ResultSetInterface<ExternalProductCatalogsProductCatalog>|false deleteMany(iterable $entities, array $options = [])
 * @method iterable<ExternalProductCatalogsProductCatalog>|ResultSetInterface<ExternalProductCatalogsProductCatalog> deleteManyOrFail(iterable $entities, array $options = [])
 *
 * @mixin TimestampBehavior
 */
class ExternalProductCatalogsProductCatalogsTable extends Table
{
    /**
     * Initialize method
     *
     * @param array<string, mixed> $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void
    {
        parent::initialize($config);

        $this->setTable('external_product_catalogs_product_catalogs');
        $this->setDisplayField('external_product_catalog_id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->setEntityClass(
            Configure::read('CakeProducts.ExternalProductCatalogsProductCatalogs.entity', 'CakeProducts\Model\Entity\ExternalProductCatalogsProductCatalog')
        );
        $this->belongsTo('ExternalProductCatalogs', [
            'className' => 'CakeProducts.ExternalProductCatalogs',
//            'foreignKey' => 'external_product_catalog_id',
            'joinType' => 'INNER',
        ]);
        $this->belongsTo('ProductCatalogs', [
            'className' => 'CakeProducts.ProductCatalogs',
//            'foreignKey' => 'product_catalog_id',
            'joinType' => 'INNER',
        ]);
    }

    /**
     * Default validation rules.
     *
     * @param Validator $validator Validator instance.
     * @return Validator
     */
    public function validationDefault(Validator $validator): Validator
    {
        $validator
            ->uuid('external_product_catalog_id')
            ->notEmptyString('external_product_catalog_id');

        $validator
            ->uuid('product_catalog_id')
            ->notEmptyString('product_catalog_id');

        $validator
            ->boolean('enabled');

        return $validator;
    }

    public function beforeMarshal(EventInterface $event, ArrayObject $data, ArrayObject $options)
    {
        if (!isset($data['enabled'])) {
            $data['enabled'] = false;
        }
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param RulesChecker $rules The rules object to be modified.
     * @return RulesChecker
     */
    public function buildRules(RulesChecker $rules): RulesChecker
    {
//        $rules->add($rules->existsIn(['external_product_catalog_id'], 'ExternalProductCatalogs'), ['errorField' => 'external_product_catalog_id']);
//        $rules->add($rules->existsIn(['product_catalog_id'], 'ProductCatalogs'), ['errorField' => 'product_catalog_id']);

        return $rules;
    }
}
