<?php
declare(strict_types=1);

namespace CakeProducts\Model\Table;

use Cake\Core\Configure;
use Cake\Datasource\EntityInterface;
use Cake\Datasource\ResultSetInterface;
use Cake\ORM\Query\SelectQuery;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use CakeProducts\Model\Entity\ProductCatalog;
use Closure;
use Psr\SimpleCache\CacheInterface;

/**
 * ProductCatalogs Model
 *
 * @method ProductCatalog newEmptyEntity()
 * @method ProductCatalog newEntity(array $data, array $options = [])
 * @method array<ProductCatalog> newEntities(array $data, array $options = [])
 * @method ProductCatalog get(mixed $primaryKey, array|string $finder = 'all', CacheInterface|string|null $cache = null, Closure|string|null $cacheKey = null, mixed ...$args)
 * @method ProductCatalog findOrCreate($search, ?callable $callback = null, array $options = [])
 * @method ProductCatalog patchEntity(EntityInterface $entity, array $data, array $options = [])
 * @method array<ProductCatalog> patchEntities(iterable $entities, array $data, array $options = [])
 * @method ProductCatalog|false save(EntityInterface $entity, array $options = [])
 * @method ProductCatalog saveOrFail(EntityInterface $entity, array $options = [])
 * @method iterable<ProductCatalog>|ResultSetInterface<ProductCatalog>|false saveMany(iterable $entities, array $options = [])
 * @method iterable<ProductCatalog>|ResultSetInterface<ProductCatalog> saveManyOrFail(iterable $entities, array $options = [])
 * @method iterable<ProductCatalog>|ResultSetInterface<ProductCatalog>|false deleteMany(iterable $entities, array $options = [])
 * @method iterable<ProductCatalog>|ResultSetInterface<ProductCatalog> deleteManyOrFail(iterable $entities, array $options = [])
 */
class ProductCatalogsTable extends Table
{
    /**
     * Initialize method
     *
     * @param array<string, mixed> $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void
    {
        parent::initialize($config);

        $this->setTable('product_catalogs');
        $this->setDisplayField('name');
        $this->setPrimaryKey('id');

        $this->setEntityClass(
            Configure::read('CakeProducts.ProductCatalogs.entity', 'CakeProducts\Model\Entity\ProductCatalog')
        );
        $this->hasMany('ProductCategories', [
            'className' => 'CakeProducts.ProductCategories',
        ]);
        $this->belongsToMany('ExternalProductCatalogs', [
            'through' => 'ExternalProductCatalogsProductCatalogs',
            'className' => 'CakeProducts.ExternalProductCatalogs',
        ]);
    }

    /**
     * Default validation rules.
     *
     * @param Validator $validator Validator instance.
     * @return Validator
     */
    public function validationDefault(Validator $validator): Validator
    {
        $validator
            ->scalar('name')
            ->maxLength('name', 255)
            ->requirePresence('name', 'create')
            ->notEmptyString('name')
            ->add('name', 'unique', ['rule' => 'validateUnique', 'provider' => 'table']);

        $validator
            ->scalar('catalog_description')
            ->maxLength('catalog_description', 255)
            ->allowEmptyString('catalog_description');

        $validator
            ->boolean('enabled')
            ->requirePresence('enabled', 'create')
            ->notEmptyString('enabled');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param RulesChecker $rules The rules object to be modified.
     * @return RulesChecker
     */
    public function buildRules(RulesChecker $rules): RulesChecker
    {
        $rules->add($rules->isUnique(['name']), ['errorField' => 'name']);

        return $rules;
    }
}
